var xml = require('libxmljs'),
    asnet = require('../io/net'),
	load = asnet.loadRequest,
    URLRequest = asnet.Request,
	Url = require('url');
	

exports.parse = parseFeed;
exports.load  = loadFeed;


require('./date.js');

function toJSON(node,stack)
{
	var children,
	    name = node.name(),
	    nodeChildren = node.childNodes();
	
	if(!(children = stack[name]))
	{
		stack[name] = children = [];
	}
	var n = nodeChildren.length;
	
	
	stack = {};
	
	attrsToJSON(node,stack);
	
	if(n == 0)
	{
		if(node.text() && node.text() != '')
		stack.nodeValue = node.text();
		children.push(stack);
	}
	else
	{
		for(var i = 0, n = nodeChildren.length; i < n; i++)
		{
			toJSON(nodeChildren[i],stack);
			
			
			children.push(stack);
		}
	}
}

function attrsToJSON(node,stack)
{
	if(node.attributes)
	{
	  var attrs = node.attrs();
  	for(var i = 0, n = ttrs.length; i < n; i++)
  	{
  		var attr = attrs[i];
  		
  		stack['@'+attr.name()] = attr.value();
  	}	
	}
}



function parseAtomEntry(node)
{
	var item = {},
	    children = node.childNodes();
	
	
	for(var i = 0, n = children.length; i < n; i++)
	{
		var childNode = children[i],
			name = childNode.name();
			
		switch(name)
		{
			case 'title':
				item.title = childNode.text();
				
			break;
			case 'link':
				item.link = childNode.attr('href').value();
			break;
			case 'published':
				item.published = new Date(childNode.text());
				
				if(!item.published.getTime())
				{
					item.published = Date.eval(childNode.text()); 
				}
			break;
			case 'dcdate':
				//example :2011-03-04T14:21:00 00:00
				item.published = new Date(childNode.text().split(' ').shift())
			break;
			case 'summary':
			case 'content':               
				item.description = childNode.text();
			break;         
			case 'contentencoded':        
				item.contentEncoded = childNode.text();
			break;
			default:
				if(name.substr(0,1) == "_")
				{
					if(!item.metadata)
					{
						item.metadata = {};
					}
					
					toJSON(childNode,item.metadata);
				}
			break;
				
		}
	}
	
	return item;
}

function parseAtom(node)
{
	var channelNode
	
	var feed = {},
	channel = {},
	items =  [];
	channel.items = items;
	feed.channels = [channel],
	children = node.childNodes();
	
	for(var i = 0, n = children.length; i < n; i++)
	{
		var childNode = children[i],
			attr;
			
		switch(childNode.name())
		{
			case 'title':
				channel.title = childNode.text();
			break;
			case 'link':
				channel.link = childNode.attr('href').value();
			break;
			case 'subtitle':
				channel.description = childNode.text();
			break;
			case 'entry':        
				items.push(parseAtomEntry(childNode));
			break;  
			default:                                
			break;
				
		}
	}
	
	
	return feed;
}

function parseRssItem(node)
{
	var item = {},
	    childNode,
	    children = node.childNodes();
	    
	for(var i = 0, n = children.length; i < n; i++)
	{
		 childNode = children[i];
		
		var name = childNode.name();
		
		switch(name)
		{
			case 'link':
			case 'title':
			case 'description':     
				item[name] = childNode.text();
			break;         
			case 'contentencoded':        
				item.contentEncoded = childNode.text();
			break;   
			break;
			case 'pubDate':
				item.published = new Date(childNode.text());
				
				if(!item.published.getTime())
				{
					item.published = Date.eval(childNode.text()); 
				}
			break;
			case 'dcdate':
				//example :2011-03-04T14:21:00 00:00
				item.published = new Date(childNode.text().split(' ').shift());
			break;
		}
	}
	
	
	return item;
}


function parseRss10(node)
{
	return parseRss20(node);
}


function parseRss20(node)
{
	
	var feed = {},
		channels = [];
	
	
	feed.channels = channels;
	
	var children = node.childNodes();
	
	
	//channels
	for(var i = 0, n = children.length; i < n; i++)
	{
		var channel = {},
			items   = [],
			item,
			channelNode = children[i],
			itemNodeChild,
			channelChildNode,
			name;
	
		if(channelNode.name() != 'channel')
			continue;
			
      
		channel.items = items;
		
		var cChildren = channelNode.childNodes();
		
		//items
		for(var j = 0,n2 = cChildren.length; j < n2; j++)
		{

			channelChildNode = cChildren[j],
			name = channelChildNode.name();
			switch(name)
			{
				case 'title':
				case 'link':
				case 'description':
					channel[name] = channelChildNode.text();
				break;
				case 'item':
					items.push(parseRssItem(channelChildNode));
				break;		
				default:              
					if(name.substr(0,4) == "_NS_")
					{
						if(!channel.metadata)
						{
							channel.metadata = {};
						}

						toJSON(channelChildNode,channel.metadata);
					}
				break;
			}
		}
		
		channels.push(channel);
	}
	
	
	return feed;
}

function parseRss(node)
{
	switch(node.attr('version').value())
	{
		case '2.0':return parseRss20(node);
		// case '1.0':return p1arseRss10(node);
	}
	
	return null;
}



function parseFeed(source,url)
{
	
	//remove ALL namespaces since this library doesn't handle them
	// source = source.replace(/(\w+):(\w+)/g,'_NS_$1_$2');
	
	try
	{

		source = source.replace(/dc\:date/g,'dcdate'). 
				replace(/content:encoded/g,'contentencoded');
		                      


		// var root = xml.parseFromString(source).documentElement;
		
		
		//CANNOT reference .root() from here because it MIGHT cause a seg fault.
		var parsed = xml.parseXmlString(source);
		
		
		var root= parsed.root();

		var data;

		switch(root.name())
		{
			case 'feed':
			data = parseAtom(root);
			break;
			case 'rss':
			data =  parseRss(root);
			break;
		}
		
		if(!data) return null;
		
		var items = data.channels[0].items;


		var hostParts = Url.parse(url),
		root = hostParts.protocol+'//'+hostParts.host;

		//fix the links.
		for(var i = items.length; i--;)
		{
			var item = items[i];
			if(item.link && item.link.indexOf('://') == -1)
			{
				
				//replace double slashes with single slashes. some sites break from ths.
				item.link = root+item.link;//.toString().replace(/[\/+]/g,'/');
			}
			
		}
		

		return data;
	}
	catch(e)
	{
		root = undefined;
		parsed = undefined;
		console.error(("failed to parse XML for "+url+" "+e.message));
	}
	
	return null;
}

function loadFeed(url,onFeed,ttl,cacheType)
{
	url = url.replace("feed://","http://");
	
	var buffer = '';
	
	if(url.indexOf('://') == -1)
	{
		console.error((url+' cannot be loaded because it is NOT a valid URL!'));
		
		return onFeed(null);
	}
	
	load(new URLRequest(url),function(buffer)
	{
		onFeed(parseFeed(buffer,url));
	})
}



